@@ -4,7 +4,7 @@ package PONAPI::Server;
 use strict;
 use warnings;
 
-our $VERSION = '0.002006';
+our $VERSION = '0.002007';
 
 use Plack::Request;
 use Plack::Response;
@@ -20,6 +20,7 @@ use PONAPI::Utils::Names qw( check_name );
 use parent 'Plack::Component';
 
 use constant {
+    ERR_MISSING_MEDIA_TYPE   => +{ __error__ => +[ 415, "{JSON:API} No {json:api} Media-Type (Content-Type / Accept)" ] },
     ERR_MISSING_CONTENT_TYPE => +{ __error__ => +[ 415, "{JSON:API} Missing Content-Type header" ] },
     ERR_WRONG_CONTENT_TYPE   => +{ __error__ => +[ 415, "{JSON:API} Invalid Content-Type header" ] },
     ERR_WRONG_HEADER_ACCEPT  => +{ __error__ => +[ 406, "{JSON:API} Invalid Accept header" ] },
@@ -172,10 +173,17 @@ sub _ponapi_check_headers {
     my $pack = HTTP::Headers::ActionPack->new;
     my $mt   = $self->{'ponapi.mediatype'};
 
+    my $has_mediatype = 0;
+
     # check Content-Type
     my $content_type = $req->headers->header('Content-Type');
-    $wr->(ERR_MISSING_CONTENT_TYPE) unless $content_type;
-    $wr->(ERR_WRONG_CONTENT_TYPE)   unless $content_type eq $mt;
+    if ( $content_type ) {
+        $wr->(ERR_WRONG_CONTENT_TYPE) unless $content_type eq $mt;
+        $has_mediatype++;
+    }
+
+    $wr->(ERR_MISSING_CONTENT_TYPE)
+        if $req->content_length and !$content_type;
 
     # check Accept
     if ( my $accept = $req->headers->header('Accept') ) {
@@ -183,9 +191,15 @@ sub _ponapi_check_headers {
             map { ( $_->[1]->type eq $mt ) ? $_->[1] : () }
             $pack->create_header( 'Accept' => $accept )->iterable;
 
-        $wr->(ERR_WRONG_HEADER_ACCEPT)
-            if @jsonapi_accept and !( grep { $_->params_are_empty } @jsonapi_accept );
+        if ( @jsonapi_accept ) {
+            $wr->(ERR_WRONG_HEADER_ACCEPT)
+                unless grep { $_->params_are_empty } @jsonapi_accept;
+
+            $has_mediatype++;
+        }
     }
+
+    $wr->(ERR_MISSING_MEDIA_TYPE) unless $has_mediatype;
 }
 
 sub _ponapi_query_params {
@@ -243,7 +257,7 @@ sub _ponapi_query_params {
 sub _ponapi_data {
     my ( $self, $wr, $req ) = @_;
 
-    return unless $req->content_length > 0;
+    return unless $req->content_length and $req->content_length > 0;
 
     $wr->(ERR_BAD_REQ) if $req->method eq 'GET';
 
@@ -340,7 +354,7 @@ PONAPI::Server - PONAPI - Perl implementation of {JSON:API} (http://jsonapi.org/
 
 =head1 VERSION
 
-version 0.002006
+version 0.002007
 
 =head1 SYNOPSIS
 
